#!/usr/bin/env python
#
#       audio.py
#       
#       Copyright 2009 Gordon Allott <gordallott@gmail.com>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

from SDL import *
from SDL.mixer import *

from gamelib import data

class audio_context(object):
    
    inited = False
    num_channels = 0
    
    def __init__(self):
        if not self.__class__.inited:
            freq = 44100#MIX_DEFAULT_FREQUENCY
            SDL_InitSubSystem(SDL_INIT_AUDIO)
            Mix_OpenAudio(freq, MIX_DEFAULT_FORMAT, 2, 4096)
            
            opened, audio_rate, audio_format, audio_channels = Mix_QuerySpec()
            channels_s = 'mono'
            if audio_channels == 2:
                channels_s = 'stereo'
            elif audio_channels > 2:
                channels_s = 'surround'
            endian = 'LE'
            if audio_format & 0x1000:
                endian = 'BE'
            print 'Opened audio at %d Hz %d bit %s (%s) - %s' % \
            (audio_rate, audio_format & 0xff, channels_s, endian, audio_channels)
            
            Mix_VolumeMusic(96)
            
            self.__class__.num_channels = Mix_AllocateChannels(32)
            self.__class__.inited = True
            
    def play_music(self, filename):
        self.__class__.current_music = Mix_LoadMUS(data.filepath('audio', 'music', filename))
        Mix_FadeInMusic(self.__class__.current_music, 0, 1000)
        Mix_HookMusicFinished(self.callback_music_finished)
        self.do_loop = True
        
    def stop_music(self):
        Mix_FadeOutMusic(1000)
        self.do_loop = False
    
    def callback_music_finished(self):
        if self.do_loop:
            Mix_FadeInMusic(self.__class__.current_music, 0, 1000)
        else:
            Mix_FreeMusic(self.callback_music)


    def play_sample(self, sample, loop=0):
        """ plays the given sample, will find a channel to play it on
        returns an id, the id is needed to stop the sample playing and such
        loop of -1 = loop forever, 0 = do not loop, 1+ = loop as many times as 
        you specify
        """
        chan = Mix_PlayChannel(-1, sample.sample, loop)
        if sample.do_3d_sound:
            Mix_SetPosition(chan, int(90 * sample.angle), int(sample.distance*255))
            
        return chan
        
    def stop_sample(self, id):
        Mix_HaltChannel(id)
            
    def update_position(self, id, sample):
        """ call this when the samples position changes and you are using 3d sound
        """
        if sample.do_3d_sound:
            Mix_SetPosition(id, int(90 * sample.angle), int(sample.distance*255))
            #Mix_Volume(id, sample.distance * 128)
            
    
        

class audio_sample(object):
    
    sample = None
    name = None
    acontext = None
    
    a_id = None
    
    do_3d_sound = False
    distance = 0.0 #loud to quiet (0.0 -> 1.0)
    angle = 0.0 #from -1.0 to 1.0 
    
    def __init__(self, filename):
        """ loads said filename as a sample """
        if not self.__class__.acontext:
            self.__class__.acontext = audio_context()
            
        self.sample = Mix_LoadWAV(data.filepath('audio', 'sample', filename))
        self.name = filename
        self.set_distance_angle(0.0, 0.0)
        self.do_3d_sound = False
        
    def set_distance_angle(self, distance, angle):
        distance = max(0.0, min(1.0, distance))
        angle = max(-1.0, min(1.0, angle))
        angle = angle * 90
        angle = 360 + angle
        self.do_3d_sound = True
        self.distance = distance
        self.angle = angle
        if self.a_id != None:
            self.__class__.acontext.update_position(self.a_id, self)
        
    def play_sample(self, loop=0):
        self.a_id = self.__class__.acontext.play_sample(self, loop=loop)
        
    def stop_sample(self):
        if self.a_id != None:
            self.__class__.acontext.stop_sample(self.a_id)
            self.a_id = None
    
        
